@@ -128,6 +128,17 @@ free_pygobject_data_list(GList *list)
 	g_list_foreach(list, (GFunc)free_pygobject_data, NULL);
 }
 
+static PyObject *
+nautilus_python_boxed_new (PyTypeObject *type, gpointer boxed, gboolean free_on_dealloc)
+{
+	PyGBoxed *self = (PyGBoxed *) type->tp_alloc (type, 0);
+	self->gtype = pyg_type_from_object ( (PyObject *) type);
+	self->boxed = boxed;
+	self->free_on_dealloc = free_on_dealloc;
+	
+	return (PyObject *) self;
+}
+
 #define METHOD_NAME "get_property_pages"
 static GList *
 nautilus_python_object_get_property_pages (NautilusPropertyPageProvider *provider,
@@ -149,7 +160,7 @@ nautilus_python_object_get_property_pages (NautilusPropertyPageProvider *provide
 								 "(N)", py_files);
 	HANDLE_RETVAL(py_ret);
 
-	HANDLE_LIST(py_ret, NautilusPropertyPage, "nautilus.PropertyPage");
+	HANDLE_LIST(py_ret, NautilusPropertyPage, "Nautilus.PropertyPage");
 	
  beach:
 	Py_XDECREF(py_ret);
@@ -222,19 +233,37 @@ nautilus_python_object_get_file_items (NautilusMenuProvider *provider,
     GList *ret = NULL;
     PyObject *py_ret = NULL, *py_files;
 	PyGILState_STATE state = pyg_gil_state_ensure();
+	PyObject *provider_version = NULL;
 	
   	debug_enter();
 
-	CHECK_OBJECT(object);
-	CHECK_METHOD_NAME(object->instance);
+	CHECK_OBJECT(object);	
 
-	CONVERT_LIST(py_files, files);
+	if (PyObject_HasAttrString(object->instance, "get_file_items_full"))
+	{
+		CONVERT_LIST(py_files, files);
+		py_ret = PyObject_CallMethod(object->instance, METHOD_PREFIX "get_file_items_full",
+									 "(NNN)",
+									 pygobject_new((GObject *)provider), 
+									 pygobject_new((GObject *)window), 
+									 py_files);
+	}
+	else if (PyObject_HasAttrString(object->instance, "get_file_items"))
+	{
+		CONVERT_LIST(py_files, files);
+		py_ret = PyObject_CallMethod(object->instance, METHOD_PREFIX METHOD_NAME,
+									 "(NN)", 
+									 pygobject_new((GObject *)window), 
+									 py_files);
+	}
+	else
+	{
+		goto beach;
+	}
 
-    py_ret = PyObject_CallMethod(object->instance, METHOD_PREFIX METHOD_NAME,
-								 "(NN)", pygobject_new((GObject *)window), py_files);
 	HANDLE_RETVAL(py_ret);
 
-	HANDLE_LIST(py_ret, NautilusMenuItem, "nautilus.MenuItem");
+	HANDLE_LIST(py_ret, NautilusMenuItem, "Nautilus.MenuItem");
 
  beach:
  	free_pygobject_data_list(files);
@@ -258,16 +287,30 @@ nautilus_python_object_get_background_items (NautilusMenuProvider *provider,
   	debug_enter();
 
 	CHECK_OBJECT(object);
-	CHECK_METHOD_NAME(object->instance);
 
-    py_ret = PyObject_CallMethod(object->instance, METHOD_PREFIX METHOD_NAME,
-								 "(NN)",
-								 pygobject_new((GObject *)window),
-								 pygobject_new((GObject *)file));
+	if (PyObject_HasAttrString(object->instance, "get_background_items_full"))
+	{
+		py_ret = PyObject_CallMethod(object->instance, METHOD_PREFIX "get_background_items_full",
+									 "(NNN)",
+									 pygobject_new((GObject *)provider),
+									 pygobject_new((GObject *)window),
+									 pygobject_new((GObject *)file));
+	}
+	else if (PyObject_HasAttrString(object->instance, "get_background_items"))
+	{
+		py_ret = PyObject_CallMethod(object->instance, METHOD_PREFIX METHOD_NAME,
+									 "(NN)",
+									 pygobject_new((GObject *)window),
+									 pygobject_new((GObject *)file));
+	}
+	else
+	{
+		goto beach;
+	}
 
 	HANDLE_RETVAL(py_ret);
 
-	HANDLE_LIST(py_ret, NautilusMenuItem, "nautilus.MenuItem");
+	HANDLE_LIST(py_ret, NautilusMenuItem, "Nautilus.MenuItem");
 	
  beach:
 	free_pygobject_data(file, NULL);
@@ -277,43 +320,10 @@ nautilus_python_object_get_background_items (NautilusMenuProvider *provider,
 }
 #undef METHOD_NAME
 
-#define METHOD_NAME "get_toolbar_items"
-static GList *
-nautilus_python_object_get_toolbar_items (NautilusMenuProvider *provider,
-										  GtkWidget 		   *window,
-										  NautilusFileInfo 	   *file)
-{
-	NautilusPythonObject *object = (NautilusPythonObject*)provider;
-    GList *ret = NULL;
-    PyObject *py_ret = NULL;
-	PyGILState_STATE state = pyg_gil_state_ensure();                                    \
-	
-  	debug_enter();
-  	
-	CHECK_OBJECT(object);
-	CHECK_METHOD_NAME(object->instance);
-
-    py_ret = PyObject_CallMethod(object->instance, METHOD_PREFIX METHOD_NAME,
-								 "(NN)",
-								 pygobject_new((GObject *)window),
-								 pygobject_new((GObject *)file));
-	HANDLE_RETVAL(py_ret);
-
-	HANDLE_LIST(py_ret, NautilusMenuItem, "nautilus.MenuItem");
-	
- beach:
- 	free_pygobject_data(file, NULL);
-	Py_XDECREF(py_ret);
-	pyg_gil_state_release(state);
-    return ret;
-}
-#undef METHOD_NAME
-
 static void
 nautilus_python_object_menu_provider_iface_init (NautilusMenuProviderIface *iface)
 {
 	iface->get_background_items = nautilus_python_object_get_background_items;
-	iface->get_toolbar_items = nautilus_python_object_get_toolbar_items;
 	iface->get_file_items = nautilus_python_object_get_file_items;
 }
 
@@ -336,7 +346,7 @@ nautilus_python_object_get_columns (NautilusColumnProvider *provider)
 
 	HANDLE_RETVAL(py_ret);
 
-	HANDLE_LIST(py_ret, NautilusColumn, "nautilus.Column");
+	HANDLE_LIST(py_ret, NautilusColumn, "Nautilus.Column");
 	
  beach:
  	Py_XDECREF(py_ret);
@@ -357,7 +367,22 @@ static void
 nautilus_python_object_cancel_update (NautilusInfoProvider 		*provider,
 									  NautilusOperationHandle 	*handle)
 {
+	NautilusPythonObject *object = (NautilusPythonObject*)provider;
+	PyGILState_STATE state = pyg_gil_state_ensure();
+	PyObject *py_handle = nautilus_python_boxed_new (_PyNautilusOperationHandle_Type, handle, FALSE);
+
   	debug_enter();
+
+	CHECK_OBJECT(object);
+	CHECK_METHOD_NAME(object->instance);
+
+    PyObject_CallMethod(object->instance,
+								 METHOD_PREFIX METHOD_NAME, "(NN)",
+								 pygobject_new((GObject*)provider),
+								 py_handle);
+
+ beach:
+	pyg_gil_state_release(state);
 }
 #undef METHOD_NAME
 
@@ -371,19 +396,35 @@ nautilus_python_object_update_file_info (NautilusInfoProvider 		*provider,
 	NautilusPythonObject *object = (NautilusPythonObject*)provider;
     NautilusOperationResult ret = NAUTILUS_OPERATION_COMPLETE;
     PyObject *py_ret = NULL;
-	PyGILState_STATE state = pyg_gil_state_ensure();                                    \
-	
+	PyGILState_STATE state = pyg_gil_state_ensure();
+	PyObject *py_handle = nautilus_python_boxed_new (_PyNautilusOperationHandle_Type, *handle, FALSE);
+
   	debug_enter();
 
 	CHECK_OBJECT(object);
-	CHECK_METHOD_NAME(object->instance);
 
-    py_ret = PyObject_CallMethod(object->instance,
-								 METHOD_PREFIX METHOD_NAME, "(N)",
-								 pygobject_new((GObject*)file));
+	if (PyObject_HasAttrString(object->instance, "update_file_info_full"))
+	{
+		py_ret = PyObject_CallMethod(object->instance,
+									 METHOD_PREFIX "update_file_info_full", "(NNNN)",
+									 pygobject_new((GObject*)provider),
+									 py_handle,
+									 pyg_boxed_new(G_TYPE_CLOSURE, update_complete, TRUE, TRUE),
+									 pygobject_new((GObject*)file));
+	}
+	else if (PyObject_HasAttrString(object->instance, "update_file_info"))
+	{
+		py_ret = PyObject_CallMethod(object->instance,
+									 METHOD_PREFIX METHOD_NAME, "(N)",
+									 pygobject_new((GObject*)file));
+	}
+	else
+	{
+		goto beach;
+	}
+	
 	HANDLE_RETVAL(py_ret);
 
-
 	if (!PyInt_Check(py_ret))
 	{
 		PyErr_SetString(PyExc_TypeError,
