@@ -45,7 +45,6 @@ extern NautilusPythonDebug nautilus_python_debug;
 #define debug_enter_args(x, y) { if (nautilus_python_debug & NAUTILUS_PYTHON_DEBUG_MISC) \
                                      g_printf("%s: entered " x "\n", __FUNCTION__, y); }
 
-
 PyTypeObject *_PyGtkWidget_Type;
 #define PyGtkWidget_Type (*_PyGtkWidget_Type)
 
@@ -76,4 +75,7 @@ PyTypeObject *_PyNautilusPropertyPage_Type;
 PyTypeObject *_PyNautilusPropertyPageProvider_Type;
 #define PyNautilusPropertyPageProvider_Type (*_PyNautilusPropertyPageProvider_Type)
 
+PyTypeObject *_PyNautilusOperationHandle_Type;
+#define PyNautilusOperationHandle_Type (*_PyNautilusOperationHandle_Type)
+
 #endif /* NAUTILUS_PYTHON_H */
